@@ -1,4 +1,4 @@
-from .resource_generator import ResourceGenerator
+from .resource_generator import ResourceGenerator, FieldValue, GeneratorParameters
 from .patient import PatientGenerator
 from .dataset import DatasetGenerator
 from .field_generator import FieldGenerator
